unit FileOper;

{

+------------------------------------------------------------------------------+
| TFileOperation Component Version 1.0                                         |
+------------------------------------------------------------------------------+
| Author: Herv Roz                                                            |
| e-mail: hroz@chez.com                                                        |
+------------------------------------------------------------------------------+
| Description:                                                                 |
|   This component encapsulates the ShFileOperation API of Microsoft Windows.  |
| Performs a copy, move, rename, or delete operation on a file system object.  |
+------------------------------------------------------------------------------+
| Properties:                                                                  |
|   Source: String that contains the names of the source files, wildcard       |
|           filename (*.*) is accepted.                                        |
|           You can include several mask separeted by a ';'.                   |                                       |
|   Destination: String that specifies the destination for the moved, copied,  |
|                 or renamed file.                                             |
|   Mode:                                                                      |
|    -foCopy	Copies the files specified by pFrom to the location            |
|               specified by pTo.                                              |
|    -foDelete	Deletes the files specified by pFrom (pTo is ignored).         |
|    -foMove	Moves the files specified by pFrom to the location             |
|               specified by pTo.                                              |
|    -foRename	Renames the files specified by pFrom.                          |
|   Options:                                                                   |
|    -fofAllowUndo	   Preserves undo information, if possible.            |
|    -fofConfirmMouse	   Not implemented.                                    |
|    -fofFilesOnly	   Performs the operation only on files if a wildcard  |
|                          filename (*.*) is specified.                        |
|    -fofMultiDestFiles    Indicates that the pTo member specifies multiple    |
|                          destination files (one for each source file) rather |
|                          than one directory where all source files are       |
|                          to be deposited.                                    |
|    -fofNoConfirmation    Responds with "yes to all" for any dialog box that  |
|                          is displayed.                                       |
|    -fofNoConfirmMkDir    Does not confirm the creation of a new directory    |
|                          if the operation requires one to be created.        |
|    -fofRenameOnCollision Gives the file being operated on a new name         |
|                          (such as "Copy #1 of...") in a move, copy,          |
|                          or rename operation if a file of the target name    |
|                          already exists.                                     |
|    -fofSilent            Does not display a progress dialog box.             |
|    -fofSimpleProgress    Displays a progress dialog box, but does not show   |
|                          the filenames.                                      |
|    -fofWantMappingHandle Not implemented.                                    |
|   Title: String to use as the title for a progress dialog box.               |
|          This member is used only if Options includes fofSimpleProgress.     |
|   AbortByUser: Value that receives True if the user aborted any file         |
|                operations before they were completed or FALSE otherwise.     |
|                                                                              |
| (*): Read only property                                                      |
+------------------------------------------------------------------------------+
| Methods:                                                                     |
|   function Execute: Performs the copy, move, rename, or delete operation.    |
|     Returns zero if successful or nonzero value if an error occurs.          |
+------------------------------------------------------------------------------+
| Events:                                                                      |
+------------------------------------------------------------------------------+

+------------------------------------------------------------------------------+
| Installation                                                                 |
+------------------------------------------------------------------------------+
| 1. Unzip the file to any directory you like, for instance                    |
|   C:\Delphi 3\Source\FileOper, ...                                           |
| 2. Start Delphi and close any projects and/or files                          |
| 3. From the menu, choose 'Components/Install Components...'                  |
| 4. In the dialog, click 'Browse'                                             |
| 5. Navigate to your chosen directory and select 'FileOper.pas'               |
| 6. Click OK the way back                                                     |
| 7. In the dialog you have now set the unit file                              |
| 8. Click OK to perform the install                                           |
+------------------------------------------------------------------------------+

+------------------------------------------------------------------------------+
| ******************************* IMPORTANT ********************************** |
+------------------------------------------------------------------------------+
|     This software is absolutely free. Use it and enjoy Delphi.               |
|     All I want is to know why and how you are using this                     |
|     software.                                                                |
|                                                                              |
|     Suggestions for improvement and fixes are always welcome,                |
|     although no guarantee is made when we'll implemement them.               |
|                                                                              |
|     If you do use this component, please drop me an email                    |
|     telling me how you used it.                                              |
|                                                                              |
|     For bug reports, suggestions, new versions etc. contact me:              |
|                                                                              |
|     e-mail: hroz@chez.com                                                    |
|     URL: http://www.chez.com/hroz                                            |
|                                                                              |
+------------------------------------------------------------------------------+

}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Dialogs, ShlObj,
  ShellApi, Forms, dsgnintf;

type TFileMode = (foCopy, foDelete, foMove, foRename);

type TFileOption = (fofAllowUndo, fofConfirmMouse, fofFilesOnly, fofMultiDestFiles,
                    fofNoConfirmation, fofNoConfirmMkDir, fofRenameOnCollision,
                    fofSilent, fofSimpleProgress, fofWantMappingHandle);

type TFileOptions = set of TFileOption;

type
  TFileOperation = class(TComponent)
  private
    { Dclarations prives }
    FAbortByUser: Boolean;
    FDestination : TFileName;
    FMode : TFileMode;
    FOptions: TFileOptions;
    FSource : TFileName;
    FTitle: string;

    function DoExecute(Mode: TFileMode): Integer;
  public
    { Dclarations publiques }
    function Execute: Integer;
  published
    { Dclarations publies }
    property AbortByUser: Boolean read FAbortByUser;
    property Destination : TFileName read FDestination write FDestination;
    property Mode : TFileMode read FMode write FMode default foCopy;
    property Options: TFileOptions read FOptions write FOptions default [fofAllowUndo, fofRenameOnCollision];
    property Source : TFileName read FSource write FSource;
    property Title: string read FTitle write FTitle;
  end;

  TSourceProperty = class(TPropertyEditor)
  public
    function GetValue: string; override;
    procedure SetValue(const Value: string); override;
    function GetAttributes: TPropertyAttributes; override;
    procedure Edit; override;
  end;

  TDestinationProperty = class(TPropertyEditor)
  public
    function GetValue: string; override;
    procedure SetValue(const Value: string); override;
    function GetAttributes: TPropertyAttributes; override;
    procedure Edit; override;
  end;

procedure register;

implementation

function TFileOperation.Execute: Integer;
begin
  Result:=DoExecute(FMode);
end;

function TFileOperation.DoExecute(Mode: TFileMode): Integer;
const
  FileMode: array [TFileMode] of Longint = (
    FO_COPY, FO_DELETE, FO_MOVE, FO_RENAME);

  FileOption: array [TFileOption] of Longint = (
    FOF_ALLOWUNDO, FOF_CONFIRMMOUSE, FOF_FILESONLY, FOF_MULTIDESTFILES,
    FOF_NOCONFIRMATION, FOF_NOCONFIRMMKDIR, FOF_RENAMEONCOLLISION,
    FOF_SILENT, FOF_SIMPLEPROGRESS, FOF_WANTMAPPINGHANDLE);
var
  ShFileOpStruct: TShFileOpStruct;

  function AllocFilterStr(const S: string): string;
  var
    P: PChar;
  begin
    Result:='';
    if S <> '' then
    begin
      Result:=S + #0;  // double null terminators
      P:=AnsiStrScan(PChar(Result), ';');
      while P <> nil do
      begin
        P^:=#0;
        Inc(P);
        P:=AnsiStrScan(P, ';');
      end;
    end;
  end;

var
  TempSource: string;
  Option: TFileOption;
begin
  FillChar(ShFileOpStruct, SizeOf(ShFileOpStruct), 0);
  with ShFileOpStruct do
  begin
    Wnd:=Application.MainForm.Handle;
    wFunc:=FileMode[FMode];
    TempSource:=AllocFilterStr(Source);
    pFrom:=PChar(TempSource);
    pTo:=PChar(Destination);
    for Option:=Low(Option) to High(Option) do
      if Option in FOptions then
        fFlags:=fFlags or FileOption[Option];
    if fofSimpleProgress in FOptions then
      lpszProgressTitle:=PChar(FTitle);
    Result := ShFileOperation(ShFileOpStruct);
    FAbortByUser:=fAnyOperationsAborted;
  end;
end;

{ TSourceProperty }

function TSourceProperty.GetValue: string;
begin
  Result:=(GetComponent(0) as TFileOperation).Source;
end;

procedure TSourceProperty.SetValue(const Value: string);
begin
  (GetComponent(0) as TFileOperation).Source:=Value;
  Modified;
end;

function TSourceProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog, paMultiSelect];
end;

procedure TSourceProperty.Edit;
var
  Dialog: TOpenDialog;
begin
  Dialog:=TOpenDialog.Create(nil);
  try
    with Dialog do
    begin
      Filter:='Tous les fichiers (*.*)|*.*';
      FileName:=(GetComponent(0) as TFileOperation).Source;
      if Execute then
        with (GetComponent(0) as TFileOperation) do
        begin
          Source:=FileName;
          Modified;
        end;
    end;
  finally
    Dialog.Free;
  end;
end;

{ TDestinationProperty }

function TDestinationProperty.GetValue: string;
begin
  Result:=(GetComponent(0) as TFileOperation).Destination;
end;

procedure TDestinationProperty.SetValue(const Value: string);
begin
  (GetComponent(0) as TFileOperation).Destination:=Value;
  Modified;
end;

function TDestinationProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog, paMultiSelect];
end;

procedure TDestinationProperty.Edit;
var
  Dialog: TOpenDialog;
begin
  Dialog:=TOpenDialog.Create(nil);
  try
    with Dialog do
    begin
      Filter:='Tous les fichiers (*.*)|*.*';
      FileName:=(GetComponent(0) as TFileOperation).Destination;
      if Execute then
        with (GetComponent(0) as TFileOperation) do
        begin
          Destination:=FileName;
          Modified;
        end;
    end;
  finally
    Dialog.Free;
  end;
end;

procedure register;
begin
  RegisterComponents('VCL', [TFileOperation]);
  RegisterPropertyEditor(TypeInfo(string), TFileOperation, 'Source', TSourceProperty);
  RegisterPropertyEditor(TypeInfo(string), TFileOperation, 'Destination', TDestinationProperty);
end;

end.
